@@ -128,7 +128,7 @@ err_noprint:
 
 	if (closure_nr_remaining(&cl) != 1) {
 		bch2_trans_unlock_long(trans);
-		bch2_wait_on_allocator(c, &cl);
+		bch2_wait_on_allocator(c, BCH_WATERMARK_normal, &cl);
 	}
 
 	return ret;
@@ -246,7 +246,8 @@ static int truncate_set_isize(struct btree_trans *trans,
 	CLASS(btree_iter_uninit, iter)(trans);
 	struct bch_inode_unpacked inode_u;
 
-	return __bch2_inode_peek(trans, &iter, &inode_u, inum, BTREE_ITER_intent, warn) ?:
+	return __bch2_inode_peek(trans, &iter, &inode_u, inum, BTREE_ITER_intent,
+				 warn ? __func__ : NULL) ?:
 		(inode_u.bi_size = new_i_size, 0) ?:
 		bch2_inode_write(trans, &iter, &inode_u);
 }
@@ -330,7 +331,8 @@ static int adjust_i_size(struct btree_trans *trans, subvol_inum inum,
 	CLASS(btree_iter_uninit, iter)(trans);
 	struct bch_inode_unpacked inode_u;
 
-	try(__bch2_inode_peek(trans, &iter, &inode_u, inum, BTREE_ITER_intent, warn));
+	try(__bch2_inode_peek(trans, &iter, &inode_u, inum, BTREE_ITER_intent,
+			      warn ? __func__ : NULL));
 
 	if (len > 0) {
 		if (MAX_LFS_FILESIZE - inode_u.bi_size < len)
@@ -426,8 +428,15 @@ case LOGGED_OP_FINSERT_shift_extents:
 		if ((ret = PTR_ERR_OR_ZERO(copy)))
 			goto btree_err;
 
-		if (insert &&
-		    bkey_lt(bkey_start_pos(k.k), src_pos)) {
+		if (snapshot != k.k->p.snapshot) {
+			ret = bch2_disk_reservation_add(c, &disk_res,
+					copy->k.size *
+					bch2_bkey_nr_ptrs_allocated(c, bkey_i_to_s_c(copy)),
+					0);
+			if (ret)
+				goto btree_err;
+		} else if (insert &&
+			   bkey_lt(bkey_start_pos(k.k), src_pos)) {
 			bch2_cut_front(c, src_pos, copy);
 
 			/* Splitting compressed extent? */
