@@ -160,7 +160,7 @@ static int bch2_set_nr_journal_buckets_loop(struct bch_fs *c, struct bch_dev *ca
 						bucket_to_sector(ca, nr - ja->nr), 1, 0));
 
 		ret = bch2_set_nr_journal_buckets_iter(ca, nr, new_fs, watermark, &cl);
-		if (ret == -BCH_ERR_open_buckets_empty)
+		if (bch2_err_matches(ret, BCH_ERR_operation_blocked))
 			ret = 0; /* wait and retry */
 
 		bch2_wait_on_allocator(c, watermark, &cl);
